#! /bin/sh
#
# This script will format the NOR and NAND partitions and then
# populate the home directory with the card afs_image contents
#

#set the current OS version number change this if you want to upgrade
#CUR_OS_VERSION="#830"
#CUR_OS_VERSION="#837"
#CUR_OS_VERSION="#841"
CUR_OS_VERSION="#842"

#intialize our path variables
INSTALL_DIR="/media/card/afs_install"
IMAGE_DIR="/media/card/afs_install/afs_image"
STATUS_DIR="/media/nor1"
BACKUP_DIR="/media/nor1/backup"
HOME_DIR="/home/root"
TEMP_DIR="/home/root/temp"
CARD_DIR="/media/card"
#   ---------------------------------------------
#   TrimDataLogs
#   Trim the datastore files to managable sizes
#   ---------------------------------------------
ContactSupportMessage()
{
    echo "Please remove SD card and"
    echo "Contact Support for assistance"
    dmesg > $INSTALL_DIR/afs_error.log #copy dmesg to card for support effort
    sync
    sleep 60
    reboot
}

#   ---------------------------------------------
#   TrimDataLogs
#   Trim the datastore files to managable sizes
#   ---------------------------------------------
TrimDataLogs()
{
    TRIMSIZE=6048000 #was 576000  # 50Hrs @ 5sec
    MAXSIZE=16777216 #16MB approx 145Hrs @ 5sec
    TEMP=$TEMP_DIR/temp

    if [ -f "$DATASTORE" ]; then
        FILESIZE=$(stat -c%s "$DATASTORE")
    else
        FILESIZE=0
    fi
    if [ -f "$DATASTORE1" ]; then
        FILESIZE1=$(stat -c%s "$DATASTORE1")
    else
        FILESIZE1=0
    fi

    if [ "$FILESIZE" -gt "$MAXSIZE" ]; then
        echo "Data logs too big... deleting"
        rm -f "$DATASTORE"
        rm -f "$DATASTORE1"
    elif [ "$FILESIZE" -gt "$TRIMSIZE" ]; then
        echo "Triming data logs to appoximately 50 hours"
        tail -c "$TRIMSIZE" "$DATASTORE" > "$TEMP"
        mv -f "$TEMP" "$DATASTORE"
        rm -f "$DATASTORE1"
    else
        #make sure datastore2 isn't too big
        TRIMREM=$(($TRIMSIZE - $FILESIZE))
        if [ "$FILESIZE1" -gt "$MAXSIZE" ]; then
            rm -f "$DATASTORE1"
        elif [ "$FILESIZE1" -gt "$TRIMREM" ]; then
            echo "Triming data logs to appoximately 50 hours"
            tail -c "$TRIMREM" "$DATASTORE1" > "$TEMP"
            mv -f "$TEMP" "$DATASTORE1"
        fi
    fi
}

#   ---------------------------------------------
#   Backup
#   Backup current home working directory
#   ---------------------------------------------
Backup()
{
    echo "Performing system backup..."

    #copy backups into $BACKUP_DIR for safe keeping
    if [ -f "$BACKUP_DIR/afs_backup.tar.gz" ]; then
        echo "Previous config backup detected moving to afs_backup.tar.gz.bak"
        mv "$BACKUP_DIR/afs_backup.tar.gz" "$BACKUP_DIR/afs_backup.tar.gz.bak"
    fi 

    if [ -f "$BACKUP_DIR/afs_datalogs.tar.gz" ]; then
        echo "Previous datalogs backup detected moving to afs_datalogs.tar.gz.bak"
        mv "$BACKUP_DIR/afs_datalogs.tar.gz" "$BACKUP_DIR/afs_datalogs.tar.gz.bak"
    fi 

  # Check for config files if there are none something bad happened and we should
  # restore from defaults
    if ! [ -d "$HOME_DIR/cfg" ]; then
        if [ -f "$BACKUP_DIR/afs_backup.tar.gz.bak" ]; then
            echo "WARNING: config files missing... restoring from backup"
            mv "$BACKUP_DIR/afs_backup.tar.gz.bak" "$BACKUP_DIR/afs_backup.tar.gz"
            tar -xzf "$BACKUP_DIR/afs_backup.tar.gz" -C "$HOME_DIR" #rebuild alternate /home/root
        else
            if [ -f "$IMAGE_DIR/sysinit.tar.gz" ]; then
                tar -xzf "$IMAGE_DIR/sysinit.tar.gz" -C "$HOME_DIR" #rebuild alternate /home/root
            else
                echo "ERROR: Config files not found"
                ContactSupportMessage
            fi
        fi
    fi

  #make the backup file
    if [ -d "$HOME_DIR/cfg" ]; then
        tar -czvf "$HOME_DIR/afs_backup.tar.gz" -C "$HOME_DIR" cfg logs afsnvram1 afsnvram2
        sync
        if [ -f "$HOME_DIR/afs_backup.tar.gz" ]; then
            cp -f "$HOME_DIR/afs_backup.tar.gz" "$BACKUP_DIR/afs_backup.tar.gz"
        fi
    fi

  #backup the datalogs but only if they are small enough
    DATASTORE=$HOME_DIR/datalogs/datastore
    DATASTORE1=$HOME_DIR/datalogs/datastore1

    if [ -f "$DATASTORE" ]; then
        TrimDataLogs
        if [ -f "$DATASTORE" ] && [ -f "$DATASTORE1" ]; then
            tar -czvf "$HOME_DIR/afs_datalogs.tar.gz" -C "$HOME_DIR/datalogs" datastore datastore1
            sync
        elif [ -f "$DATASTORE" ]; then
            tar -czvf "$HOME_DIR/afs_datalogs.tar.gz" -C "$HOME_DIR/datalogs" datastore
            sync
        fi
    fi


    if [ -f "$HOME_DIR/afs_datalogs.tar.gz" ]; then
        cp -f "$HOME_DIR/afs_datalogs.tar.gz" "$BACKUP_DIR/afs_datalogs.tar.gz"
    fi
    sync

    echo "Backup complete"
    return 0;
}

#   ---------------------------------------------
#   CopyOsToSD
#   Prepare SD card for UBOOT copy of OS
#   ---------------------------------------------
CopyOsToSD()
{
    echo "Preparing the SD card for OS upgrade"

    if [ -f "$INSTALL_DIR/uImage.bak" ]; then
        mv $INSTALL_DIR/uImage.bak $INSTALL_DIR/uImage
        if [ $? -ne 0 ]; then
            echo "ERROR: SD Card failure"
            ContactSupportMessage
        fi
    else
        echo "WARNING: uImage.bak not found"
    fi
    sync
    return 0;
}

#   ---------------------------------------------
#   CopyOsToSD
#   Prepare SD card for UBOOT copy of OS
#   ---------------------------------------------
CopyCramfsToSD()
{
    echo "Preparing the SD card for file system upgrade"

    if [ -f "$INSTALL_DIR/uCramfsDisk.bak" ]; then
        mv $INSTALL_DIR/uCramfsDisk.bak $INSTALL_DIR/uCramfsDisk
        if [ $? -ne 0 ]; then
            echo "ERROR: SD Card failure"
            ContactSupportMessage
        fi
    else
        echo "WARNING: uCramfsDisk.bak not found"
    fi
    sync
    return 0;
}

#   ---------------------------------------------
#   CopyEnvToSD
#   Prepare SD card for ubifs environment
#   ---------------------------------------------
CopyEnvToSD()
{
    if ! [ -f "$IMAGE_DIR/factory_card" ]; then
        echo "Preparing the SD card for Read/Write file system use"
        cp -f $IMAGE_DIR/setenv_ubootenv.img $INSTALL_DIR/ubootenv.img
        if [ $? -ne 0 ]; then
            echo "ERROR: SD Card failure"
            ContactSupportMessage
        fi
        sync            
    fi

    return 0;
}

#   ---------------------------------------------
#   BuildUBIFS
#   Initialize UBI Filesystem
#   ---------------------------------------------
BuildUBIFS()
{

    # run ubi_convert script to format and populate ubi file system
    echo "Upgrading file system... please stand by"
    echo "ubistarted" > $STATUS_DIR/ubistarted

    if ! [ -f "$BACKUP_DIR/afs_backup.tar.gz" ] && [ -f "$BACKUP_DIR/afs_backup.tar.gz.bak" ]; then
        echo "ERROR: afs_backup not found"
        return 0;
    fi
    
    # disconnect anything connected to mtd9
    $IMAGE_DIR/ubi-utils/ubidetach /dev/ubi_ctrl -m 9

    # erase and format mtd9 for UBIFS
    $IMAGE_DIR/ubi-utils/ubiformat -y /dev/mtd9

    # attach MTD9 to UBI, calling it "ubi0"
    $IMAGE_DIR/ubi-utils/ubiattach /dev/ubi_ctrl -m 9 -d 0

    # create the right device node, if necessary; if you already have
    # a /dev/ubi0, then make sure its major/minor number agree with
    # what is listed in /proc/devices for ubi0
    rm -f /dev/ubi0
    mknod /dev/ubi0 c `grep ubi0 /proc/devices | awk '{print$1;}'` 0
    
    # create a "volume" out of ubi0, call it "userfs"
    # (conveniently the same name given to the MTD)
    $IMAGE_DIR/ubi-utils/ubimkvol -N userfs -m /dev/ubi0

    # mount the volume
    mount -t ubifs ubi0:userfs /media/nand2

    echo "Populating file system"
    tar -xzf $IMAGE_DIR/afs_rootfs.tar.gz -C /media/nand2
    sync 

    if [ -f "$BACKUP_DIR/afs_backup.tar.gz" ]; then
        echo "Restoring config files from backup"
        tar -xzf $BACKUP_DIR/afs_backup.tar.gz -C /media/nand2/home/root
    elif [ -f "$BACKUP_DIR/afs_backup.tar.gz.bak" ]; then
        mv $BACKUP_DIR/afs_backup.tar.gz.bak $BACKUP_DIR/afs_backup.tar.gz
        echo "Restoring config files from backup"
        tar -xzf $BACKUP_DIR/afs_backup.tar.gz -C /media/nand2/home/root
    fi

    mkdir -p /media/nand2/home/root/datalogs
    if [ -f "$BACKUP_DIR/afs_datalogs.tar.gz" ]; then
        echo "Restoring datalogs from backup"
        tar -xzf $BACKUP_DIR/afs_datalogs.tar.gz -C /media/nand2/home/root/datalogs
    else
        if [ -f "$BACKUP_DIR/afs_datalogs.tar.gz.bak" ]; then
            mv $BACKUP_DIR/afs_datalogs.tar.gz.bak $BACKUP_DIR/afs_datalogs.tar.gz
            echo "Restoring datalogs from backup"
            tar -xzf $BACKUP_DIR/afs_datalogs.tar.gz -C /media/nand2/home/root/datalogs
        else
            echo "INFO: Datalogs backup not available"
            sleep 1
        fi
    fi
    sync

    echo "ubifinished" > $STATUS_DIR/ubifinished
    rm -f $STATUS_DIR/ubistarted #remove marker to show success
    sync
    echo "File system upgrade complete"

    return 0;
}


#   ---------------------------------------------
#   Cleanup
#   Cleans up the SD card to avoid unintentional reinstalls
#   ---------------------------------------------
Cleanup()
{
    # clean up from install and copy over app components
    if [ -f "$STATUS_DIR/ubistarted" ]; then
        rm -f $STATUS_DIR/ubistarted
    fi
    
    if [ -f "$STATUS_DIR/ubifinished" ]; then
        rm -f $STATUS_DIR/ubifinished
    fi

    if ! [ -f "$IMAGE_DIR/factory_card" ]; then

        echo "Restoring Install Image Files"
        if [ -f "$INSTALL_DIR/uImage" ]; then
            mv $INSTALL_DIR/uImage $INSTALL_DIR/uImage.bak 
            if [ $? -ne 0 ]; then
                echo "ERROR: SD Card failure"
                ContactSupportMessage
            fi
        fi
        
        if [ -f "$INSTALL_DIR/uCramfsDisk" ]; then
            mv $INSTALL_DIR/uCramfsdisk $INSTALL_DIR/uCramfsdisk.bak
            if [ $? -ne 0 ]; then
                echo "ERROR: SD Card failure"
                ContactSupportMessage
            fi
        fi
        
        cp -f $IMAGE_DIR/setcon_ubootenv.img $INSTALL_DIR/ubootenv.img #restore tty0 console set
        if [ $? -ne 0 ]; then
            echo "ERROR: SD Card failure"
            ContactSupportMessage
        fi
        sync
    fi

    if [ -f "$HOME_DIR/afs_datalogs.tar.gz" ]; then
        echo "Copying raw datalogs to card"
        cp -f "$HOME_DIR/afs_datalogs.tar.gz" "$CARD_DIR/afs_datalogs.tar.gz"
        if [ $? -ne 0 ]; then
            echo "ERROR: SD Card failure"
            ContactSupportMessage
        fi
        sync
    fi

    if [ -f "$HOME_DIR/afs_backup.tar.gz" ]; then
        echo "Copying backup to card"
        cp -f "$HOME_DIR/afs_backup.tar.gz" "$CARD_DIR/afs_backup.tar.gz"
        if [ $? -ne 0 ]; then
            echo "ERROR: SD Card failure"
            ContactSupportMessage
        fi
        sync
    fi

    return 0;

}

FinalUpdates()
{

# Update FileSystem drivers
    if [ -f "$IMAGE_DIR/afs_fspatch.tar.gz" ]; then
        echo "Updating File System"
        tar -xzf $IMAGE_DIR/afs_fspatch.tar.gz -C /
        sync
        echo "File System Update Complete"
    fi

# Update GFX drivers
    if [ -f "$IMAGE_DIR/afs_gfxdrivers.tar.gz" ]; then
        echo "Updating Graphics Drivers"
        /etc/init.d/rc.pvr stop
        tar -xzf $IMAGE_DIR/afs_gfxdrivers.tar.gz -C /
        sync
        /etc/init.d/rc.pvr start
        echo "Graphics Drivers Update Complete"
    fi
    
# Update powervr.ini
    if [ -f "$IMAGE_DIR/powervr.ini" ]; then
        echo "Updating powervr.ini file"
        /etc/init.d/rc.pvr stop
        cp -f $IMAGE_DIR/powervr.ini /etc/
        sync
        echo "powervr.ini file Update Complete"
    fi

# SysInit Option 
# if install card includes sysinit.tar.gz extract it into /home/root
    if [ -f "$IMAGE_DIR/sysinit.tar.gz" ]; then
        echo "sysinitstarted" > $INSTALL_DIR/sysinitstarted
        sync
        echo "Initializing default data from sysinit.tar.gz"
        tar -xzf $IMAGE_DIR/sysinit.tar.gz -C $HOME_DIR
        rm -f $INSTALL_DIR/sysinitstarted
        sync
    fi
# AF4000.AFE AFSMON.AFE and AFSWM.AFE are copied by runafs    

# AFEXEC.AFE
    if [ -f "$INSTALL_DIR/AFEXEC.AFE" ]; then
        echo "Upgrading AFEXEC.AFE From SD Card"
        cp -af $INSTALL_DIR/AFEXEC.AFE $HOME_DIR/AFEXEC.AFE
        sync
        sleep 1
    fi

# SRVXM.AFE
    if [ -f "$INSTALL_DIR/SRVXM.AFE" ]; then
        echo "Upgrading SRVXM.AFE From SD Card"
        cp -af $INSTALL_DIR/SRVXM.AFE $HOME_DIR/SRVXM.AFE
        sync
        sleep 1
    fi
# SRVFISB.AFE
    if [ -f "$INSTALL_DIR/SRVFISB.AFE" ]; then
        echo "Upgrading SRVFISB.AFE From SD Card"
        cp -af $INSTALL_DIR/SRVFISB.AFE $HOME_DIR/SRVFISB.AFE
        sync
        sleep 1
    fi
# WMM.COF
    if [ -f "$INSTALL_DIR/WMM.COF" ]; then
        echo "Upgrading WMM.COF From SD Card"
        cp -af $INSTALL_DIR/WMM.COF $HOME_DIR/map/WMM.COF
        sync
        sleep 1
    fi
# WBDEF.AFZ
    if [ -f "$INSTALL_DIR/WBDEF.AFZ" ]; then
        echo "Upgrading WBDEF.AFZ From SD Card"
        cp -af $INSTALL_DIR/WBDEF.AFZ $HOME_DIR/WBDEF.AFZ
        sync
        sleep 1
    fi


    return 0;
}

#   ---------------------------------------------
#   CopyEnvUbiBoot
#   Writes UBI environment to FLASH
#   ---------------------------------------------
CopyEnvUbiBoot()
{
    ENVFILE=ubibootenv.img   
    
    echo "Copying Bootloader settings from SD"
    
    if [ -f "$IMAGE_DIR/$ENVFILE" ]; then
        cp -f "$IMAGE_DIR/$ENVFILE" "$TEMP_DIR/$ENVFILE"
        if [ $? -ne 0 ]; then
            echo "ERROR: SD Card failure"
            ContactSupportMessage
        fi
        sync

    else
        echo "ERROR: settings file $IMAGE_DIR/$ENVFILE not found"
        ContactSupportMessage
    fi        

}

#   ---------------------------------------------
#   UpdateEnvUbiBoot
#   Writes UBI environment to FLASH
#   ---------------------------------------------
UpdateEnvUbiBoot()
{
    ENVFILE=ubibootenv.img   
    
    echo "Writing Bootloader settings to FLASH"
    
    if [ -f "$TEMP_DIR/$ENVFILE" ]; then
        FILESIZE=$(stat -c%s "$TEMP_DIR/$ENVFILE")
        echo "Erasing FLASH for Bootloader settings"
        $IMAGE_DIR/ubi-utils/flash_erase /dev/mtd1
        echo "Writing Bootloader settings to FLASH"
        dd if="$TEMP_DIR/$ENVFILE" of=/dev/mtdblock1 count="$FILESIZE"
        echo "Done"
    else
        echo "ERROR: Cannot find $TEMP_DIR/$ENVFILE"
        ContactSupportMessage
    fi
}

#   ---------------------------------------------
#   CopyEnvNorBoot
#   Writes NorBoot environment to FLASH
#   ---------------------------------------------
CopyEnvNorBoot()
{
    ENVFILE=norbootenv.img   
    echo "Copying Bootloader settings from SD"
    
    if [ -f "$IMAGE_DIR/$ENVFILE" ]; then
       
        cp -f "$IMAGE_DIR/$ENVFILE" "$TEMP_DIR/$ENVFILE"
        if [ $? -ne 0 ]; then
            echo "ERROR: SD Card failure"
            ContactSupportMessage
        fi
        sync

    else
        echo "ERROR: settings file $IMAGE_DIR/$ENVFILE not found"
        ContactSupportMessage
    fi        
      
}
#   ---------------------------------------------
#   UpdateEnvNorBoot
#   Writes NorBoot environment to FLASH
#   ---------------------------------------------
UpdateEnvNorBoot()
{
    ENVFILE=norbootenv.img   
    echo "Writing Bootloader settings to FLASH"

    if [ -f "$TEMP_DIR/$ENVFILE" ]; then
        FILESIZE=$(stat -c%s "$TEMP_DIR/$ENVFILE")
        echo "Erasing FLASH for Bootloader settings"
        $IMAGE_DIR/ubi-utils/flash_erase /dev/mtd1
        echo "Writing Bootloader settings to FLASH"
        dd if="$TEMP_DIR/$ENVFILE" of=/dev/mtdblock1 count="$FILESIZE"
        echo "Done"
    else
        echo "ERROR: Cannot find $TEMP_DIR/$ENVFILE"
        ContactSupportMessage
    fi
}


#   ---------------------------------------------
#   CopyKernel
#   Writes Kernel to FLASH
#   ---------------------------------------------
CopyKernel()
{

    echo "Copying Kernel image from SD"

    if [ -f "$INSTALL_DIR/uImage.bak" ]; then 
        UIMAGE=uImage.bak
    elif [ -f "$INSTALL_DIR/uImage" ]; then
        UIMAGE=uImage
    else
        echo "ERROR: Kernel image not found"
        ContactSupportMessage
    fi

    cp -f "$INSTALL_DIR/$UIMAGE" "$TEMP_DIR/$UIMAGE"
    if [ $? -ne 0 ]; then
        echo "ERROR: SD Card failure"
        ContactSupportMessage
    fi
    sync

}      

#   ---------------------------------------------
#   UpdateKernel
#   Writes Kernel to FLASH
#   ---------------------------------------------
UpdateKernel()
{
    echo "Writing Kernel image to FLASH"

    if [ -f "$TEMP_DIR/$UIMAGE" ]; then
        FILESIZE=$(stat -c%s "$TEMP_DIR/$UIMAGE")
        echo "Erasing FLASH for Kernel image"
        $IMAGE_DIR/ubi-utils/flash_erase /dev/mtd4
        echo "Writing $FILESIZE byte Kernel to FLASH "
        dd if="$TEMP_DIR/$UIMAGE" of=/dev/mtdblock4 count="$FILESIZE"
        # Update GFX drivers
        if [ -f "$IMAGE_DIR/afs_gfxdrivers.tar.gz" ]; then
            echo "Updating Graphics Drivers"
            /etc/init.d/rc.pvr stop
            tar -xzf $IMAGE_DIR/afs_gfxdrivers.tar.gz -C /
            sync
            #/etc/init.d/rc.pvr start
            echo "Graphics Drivers Update Complete"
        fi
        echo "Done"
    else
        echo "ERROR: Cannot find $TEMP_DIR/$UIMAGE"
        ContactSupportMessage
    fi

}

#   ---------------------------------------------
#   CopyCramFS
#   Writes uCramfsdisk to FLASH
#   ---------------------------------------------
CopyCramfs()
{

    echo "Copying filesys image from SD"

    if [ -f "$INSTALL_DIR/uCramfsdisk.bak" ]; then 
        CRAMFS=uCramfsdisk.bak
    elif [ -f "$INSTALL_DIR/uCramfsdisk" ]; then
        CRAMFS=uCramfsdisk
    else
        echo "ERROR: Filesys image not found"
        ContactSupportMessage
    fi

    cp -f "$INSTALL_DIR/$CRAMFS" "$TEMP_DIR/$CRAMFS"
    if [ $? -ne 0 ]; then
        echo "ERROR: SD Card failure"
        ContactSupportMessage
    fi
    sync

}

#   ---------------------------------------------
#   UpdateCramFS
#   Writes uCramfsdisk to FLASH
#   ---------------------------------------------
UpdateCramfs()
{
     
    echo "Writing filesys image to FLASH"

    if [ -f "$TEMP_DIR/$CRAMFS" ]; then
        FILESIZE=$(stat -c%s "$TEMP_DIR/$CRAMFS")
        echo "Erasing FLASH for filesys image"
        $IMAGE_DIR/ubi-utils/flash_erase /dev/mtd5
        echo "Writing $FILESIZE byte filesys image to FLASH"
        dd if="$TEMP_DIR/$CRAMFS" of=/dev/mtdblock5 count="$FILESIZE"
        echo "Done"
    else
        echo "ERROR: Cannot find $TEMP_DIR/$CRAMFS"
        ContactSupportMessage
    fi

}

#   ---------------------------------------------
#   CheckDirectories
#   Make sure basic directory structure is in place
#   ---------------------------------------------
CheckDirectories()
{
    if ! [ -d "$HOME_DIR/cfg" ]; then
        mkdir -p "$HOME_DIR/cfg"
    fi

    if ! [ -d "$HOME_DIR/datalogs" ]; then
        mkdir -p "$HOME_DIR/datalogs"
    fi

    if ! [ -d "$HOME_DIR/logs" ]; then
        mkdir -p "$HOME_DIR/logs"
    fi

    if ! [ -d "$HOME_DIR/sounds" ]; then
        mkdir -p "$HOME_DIR/sounds"
    fi

    if ! [ -d "$HOME_DIR/map" ]; then
        mkdir -p "$HOME_DIR/map"
    fi

    if ! [ -d "$HOME_DIR/wx/xm/srv" ]; then
        mkdir -p "$HOME_DIR/wx/xm/srv"
    fi

    if ! [ -d "$HOME_DIR/wx/fisb/srv" ]; then
        mkdir -p "$HOME_DIR/wx/fisb/srv"
    fi

}



#   ---------------------------------------------
#   ---------------------------------------------
#   The work starts here
#   ---------------------------------------------
#   ---------------------------------------------

echo "Running Advanced Flight Systems Upgrade"
UBIFS="UBIFS"
CRAMFS="CRAMFS"
filesys=$CRAMFS

# move to /
cd /

#get the file system type
for x in $(cat /proc/cmdline); do
        case $x in
            root=ubi0:userfs)
            filesys=$UBIFS
            ;;
        esac
done

#mount NOR FLASH partition for backups
mount -t jffs2 /dev/mtdblock6 /media/nor1
if ! [ -d $BACKUP_DIR ]; then
    mkdir -p "$BACKUP_DIR"
fi

if ! [ -d $TEMP_DIR ]; then
    mkdir -p "$TEMP_DIR"
fi

#get the running os version '#830'
osver=`uname -v | cut -c1-4`
echo "OS Version: $osver detected"

if [ "&osver" < "$CUR_OS_VERSION" ]; then #older than version 8.02.18 software
    echo "Additional upgrade steps required please contact AFS Support"
    sleep 10
    exit 0;
fi

if [ "$filesys" == "$UBIFS" ]
then
    echo "Read/Write file system detected"
else
    echo "Write Only file system detected"
fi

# Install Steps
# 1: Check FS type
# 2: Check OS type
# 3: Update OS
# 4: Update FS
# 5: Final updates

# Options here are:
# A: OS and FS are up to date then:
#    A1: Cleanup SD card
#    A2: Run final updates (installation exit point)
 
# B: FS is UBIFS then:
#    B1: OS Update needed:
#        B1a: Copy new OS to SD
#        B1b: Reboot (start over)

# C: FS is CRAMDISK
#    C1: OS Update needed: 
#        C1a: Copy uImage to SD
#        C1b: Copy uCramfsdisk to SD
#        C1c: Reboot (start over)
#    C2: OS current:
#        C2a: Backup current settings and datalogs onto SD card
#        C2b: Build UBIFS
#        C2c: Populate root file system
#        C2d: Restore settings and datalogs from SD backups
#        C2b: Reset enviroment to boot UBIFS
#        C2c: Reboot (start over)

if [ "$filesys" == "$UBIFS" ]; then # kernel update is only option
    if [ "$osver" == "$CUR_OS_VERSION" ]; then #option A
        echo "OS Version is up to date"
        Backup
        Cleanup        #cleanup SD card after OS upgrade
        rm -f $INSTALL_DIR/afs_error.log
        CheckDirectories
    else                                       #option B
        echo "OS Upgrade to $CUR_OS_VERSION required"
        CopyKernel
        CopyEnvUbiBoot
        UpdateKernel
        UpdateEnvUbiBoot

        if [ -f "$IMAGE_DIR/factory_card" ]; then
            echo "Installation complete, please remove SD card"
            sleep 300
        else
            echo "Reboot required, please stand by"
            #CopyOsToSD     #load SD w/ OS and reboot
            #CopyEnvToSD    #ensure the environment is set correctly
            #echo "Reboot required, next step duration 3-5 min"
            sleep 10
        fi
        reboot
        exit 0;
    fi
else      #need to run the full upgrade   
    echo "File system upgrade required"
    if [ "$osver" == "$CUR_OS_VERSION" ]; then #option C2
        echo "OS Version is up to date"
        Backup            #make a backup
        Cleanup           #reset the card to start all over again
        BuildUBIFS        #update filesystem, load SD w/ Enviroment, and reboot
        if [ -f "$STATUS_DIR/ubifinished" ]; then
            rm -f $STATUS_DIR/ubifinished
            CopyEnvUbiBoot            
            UpdateEnvUbiBoot
            echo "Reboot required, please stand by"
            sleep 5
        else
            echo "Installation error (UBIFS_ERROR) please contact AFS support"
            Cleanup
            sync
            ContactSupportMessage
        fi
    else  #wrong OS update both OS and Cramfs  #option C1
        echo "OS Upgrade needed to complete file system upgrade"
        CopyKernel
        CopyCramfs
        CopyEnvNorBoot
        
        UpdateKernel
        UpdateCramfs
        UpdateEnvNorBoot

        echo "Reboot required, please stand by"
        sleep 5
    fi
    reboot
    exit 0;
fi

FinalUpdates

echo "Advanced Flight Systems Upgrade Complete"

: exit 0;
